library(shiny)

numsim <- 2000
tdr <- paste("<TD align=\"right\">")
red <- paste("<font color=red>")
black <- paste("<font color=black>")
plus1 <- paste("<font size=\"+1\">")
shinyServer(function(input, output) {
       
  data <- reactive({
      x <- matrix(rnorm(input$n*numsim,input$mu,input$sigma),nrow=numsim)
      xbar <- apply(x,1,mean)
      s <- apply(x,1,sd)
      z <- qnorm(1-(1-as.numeric(input$alpha)/100)/2)
      low <- xbar-z*s/sqrt(input$n)
      high <- xbar+z*s/sqrt(input$n)
      check <- ifelse(low>input$mu | high<input$mu,"Bad","Good")
      A <- data.frame('Sample Mean'=round(xbar,1),'Standard Deviation'=round(s,1),
                    Low=round(low,1),High=round(high,1),Check=check)
      A
    })

    html_data <- reactive({
        x <- data()
        x1 <- sprintf("%.1f",sum(x[[5]]=="Good")/length(x[[5]])*100)
        a1 <- paste("<h4>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;Percentage of good intervals:",x1,"%</h4>",collapse="")
        x2 <- sprintf("%.1f",mean(x[,4]-x[,3]))
        a2 <- paste("<h4>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;Mean length of intervals:",x2,"</h4>",collapse="")
        x3 <- sprintf("%.1f",input$mu)
        a3 <- paste("<h4>&nbsp;&nbsp;&nbsp;&nbsp; &nbsp;&nbsp;Population mean &mu;:",x3,"</h4>",collapse="")
        
        a4 <- c("<h4>&nbsp;</h4>","<h4>First 100 Intervals (of 2000):</h4>")
        n <- 100
        m <- dim(x)[2]
        top <- "<TABLE BORDER width=400>"
        nameline <- "<TR><TH>Sample Mean &nbsp;</TH>
                         <TH>Std. Dev &nbsp; </TH>
                         <TH>Left End &nbsp; </TH>
                         <TH>Right End &nbsp; </TH>
                         <TH>Good Interval? &nbsp; </TH></TR>"

        middle <- rep("",n)
        for (j in 1:n) {
             if(x[j,5]=="Good") 
                  middle[j] <- paste("<TR>",tdr,sprintf("%.1f", x[j, 1]),"&nbsp;</TD>", 
                                tdr,sprintf("%.1f", x[j, 2]),"&nbsp;</TD>",
                                tdr,sprintf("%.1f", x[j, 3]),"&nbsp;</TD>",
                                tdr,sprintf("%.1f", x[j, 4]),"&nbsp;</TD>",
                                tdr,x[j, 5],"&nbsp;</TD>",
                                "</TR>",collapse="")
             else  
                  middle[j] <- paste("<TR>",tdr,sprintf("%.1f",x[j, 1]),"&nbsp;</TD>", 
                                tdr,sprintf("%.1f", x[j, 2]),"&nbsp;</TD>",
                                tdr,red,sprintf("%.1f",x[j, 3]),"&nbsp;</TD>",
                                tdr,red,sprintf("%.1f",x[j, 4]),"&nbsp;</TD>",
                                tdr,red,plus1,x[j, 5],"&nbsp;</TD>",
                                "</TR>",collapse="")
        }
        bottom <- "</Table>"
        c(a1,a2,a3,a4,top,nameline,middle,bottom)
   })    
      
   output$table1 <- renderText({
        html_data()
   })      
   
   output$table2 <- renderText({
      n <- input$n
      for(i in 0:input$step) {
        if(i==0) line <- " "
        else { 
        x <- sort(round(rnorm(n,input$mu,input$sigma),1))
        line <- "<h4>Ordered Data</h4>"
        line[2] <- paste("<h4>",paste(x,"&nbsp;",collapse=" "),"</h4>")
        xbar <- round(mean(x),2)
        s <- round(sd(x),2)
        alpha <- 1-as.numeric(input$alpha)/100
        crit <- round(qt(1-alpha/2,n-1),3)
        line[3] <- "<hr>"               
        line[4] <- "<h4>Summary Statistics</h4>"       
        line[5] <- paste("<h4>n = ",length(x),"</h4>")       
        line[6] <- paste("<h4>X&#773; = ",xbar,"</h4>")       
        line[7] <- paste("<h4>s = ",s,"</h4>") 
        line[8] <- paste("<h4>t<sub>&alpha;/2,n-1</sub> = 
              t<sub>",alpha/2,",",n-1,"</sub> = ",crit,"</h4>")
        line[9] <- "<hr>" 
        line[10] <- "<h4>Confidence Interval Calculation</h4>"                                 
        line[11] <- paste("<h4>X&#773; &plusmn; t<sub>&alpha;/2,n-1</sub>s/&radic;n =</h4>")       
        line[12] <- paste("<h4>",xbar," &plusmn; ",crit,"*",s,"/&radic;",n," =</h4>")       
        line[13] <- paste("<h4>",xbar," &plusmn; ",round(crit*s/sqrt(n),2),"</h4>")       
        line[14] <- "<hr>" 
        line[15] <- paste("<h4>A ",input$alpha,"% Confidence Interval for the population mean is</h4>")
        line[16] <- paste("<h4>(",round(xbar-crit*s/sqrt(n),2),",",
                          round(xbar+crit*s/sqrt(n),2),")</h4>")                                         
        
        if(xbar-crit*s/sqrt(n)<input$mu & input$mu<xbar+crit*s/sqrt(n)) 
           line[17] <- paste("<h4>Population mean &mu;=",input$mu," is in the confidence interval, so interval is good</h4>") 
        else      
           line[17] <- paste("<h4>Population mean &mu;=",input$mu," is ",red,"NOT",black," in the confidence interval, so interval is bad</h4>") 
        line[18] <- paste("<h4>Length of Interval is ",round(xbar+crit*s/sqrt(n),2),"-",
                          round(xbar-crit*s/sqrt(n),2)," = ",
                          round(2*crit*s/sqrt(n),2),"</h4>") 
        line[19] <- paste("<h4>Mean Length of many such Intervals is ",round(2*crit*input$sigma/sqrt(n),2),"</h4>")                                                                            
    }         
    }         
    line
    
        
   })    
        
   output$appinfo <- renderText({ 
      HTML("<br>&nbsp;<br>&nbsp;<br>&nbsp;<br>
      <a target=\"_blank\" href=\"http://academic.uprm.edu/wrolke/myapps1.html#confint\"><h4>What to do</h4></a><br>
      <h6>Written by: Dr. Wolfgang Rolke<br>Created: 11/22/2014</h6>")
      
  })  
  
  output$explainText <- renderText({ 
        "<h5>Explanation: This demo illustrates the concept of
             a confidence interval<br> </h5>"
        
  })
           
})

